import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GradientPaint;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.geom.Dimension2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.util.Random;

import javax.swing.*;
import javax.swing.SpringLayout;

import com.mindfusion.diagramming.*;

public class MainFrame extends JFrame
{
	public MainFrame()
	{
		super("MindFusion.Diagramming Sample: Custom Draw");
		initFrame();
		initDiagram();
	}

	private void initDiagram()
	{
		diagram.setBackBrush(new SolidBrush(new Color(245, 245, 245)));
		diagram.setDynamicLinks(true);
		diagram.setDefaultShape(Shape.fromId("Ellipse"));
		diagram.setShapeHandlesStyle(HandlesStyle.Invisible);
		diagram.setLinkCustomDraw(CustomDraw.Full);
		diagram.setAllowSelfLoops(false);
		diagram.setAlignToGrid(false);
		diagram.setSelectAfterCreate(false);

		diagram.addDiagramListener(new DiagramAdapter()
		{
			public void drawLink(DrawLinkEvent e) 
			{
				MainFrame.this.drawLink(e);
			}

			public void linkCreated(LinkEvent e)
			{
				initLink(e);
			}

			public void clicked(DiagramEvent e)
			{
				Point2D.Float location = e.getMousePosition();
				location.x -= 8;
				location.y -= 8;

				Dimension2D size = new Dimension(15, 15);

				ShapeNode shape = diagram.getFactory().createShapeNode(location, size);
				shape.setBrush(new SolidBrush(new Color(
					_r.nextInt(255), _r.nextInt(255), _r.nextInt(255))));
			}
		});
	}
	
	private void animate()
	{
		timer.start();
	}
	
	private void initLink(LinkEvent e)
	{
		// Zero the progress
		// (the progress is considered to vary from 0 to 100 in the sample)
		e.getLink().setWeight(1);

		// Set progress type
		e.getLink().setTag(cmBxLinkShape.getSelectedItem().toString().toUpperCase());
	}

	private void drawLink(DrawLinkEvent e)
	{
		if (e.getShadow())
		{
			// We are not drawing link shadows
			return;
		}

		Paint paint = e.getGraphics().getPaint();
		
		Point2D.Float a = (Point2D.Float)e.getPoints().get(0).clone();
		Point2D.Float b = (Point2D.Float)e.getPoints().get(e.getPoints().size() - 1).clone();

		if (a.equals(b))
		{
			// 0-length link, skip drawing
			return;
		}

		// If everything goes as planned, the arrow's type
		// should be found in the arrow's tag (where we put it
		// when the arrow was created).
		// There is a good chance that the arrow is not yet
		// considered created if the arrow is still being
		// dragged by the used from one box to another. In this case
		// consider the arrow's type to be Flat.
		String type = LinkFillType.FLAT;
		if (e.getLink().getTag() instanceof String)
			type = (String)e.getLink().getTag();

		// The fill color of the origin
		Color oc = e.getLink().getOrigin() == null ?
			Color.black : ((SolidBrush)e.getLink().getOrigin().getBrush()).getColor();
		// The fill color of the destination. The destination
		// can be null if the arrow is not yet created
		Color dc = e.getLink().getDestination() == null ?
			Color.black : ((SolidBrush)e.getLink().getDestination().getBrush()).getColor();

		// In order to display a rectangle instead of a simple line,
		// we need to find four points. For this purpose we
		// perform the following transformations
		//Float an = new Float(0), r = new Float(0);
		PolarSystemInfo polarInfo = new PolarSystemInfo();
		Point2D.Float p1 = new Point2D.Float();
		Point2D.Float p2 = new Point2D.Float();
		Point2D.Float p3 = new Point2D.Float();
		Point2D.Float p4 = new Point2D.Float();

		// Find two points around the origin control point
		// here: Convert.dekartToPolar(a, b, an, r);
		Convert.cartesianToPolar(a, b, polarInfo);
		Convert.polarToCartesian(a, polarInfo.angle + 105, 1.2f, p1);
		Convert.polarToCartesian(a, polarInfo.angle - 105, 1.2f, p2);

		// At this point in 'r' we have the length of the arrow
		// expressed in the current graphics unit (millimeters by default),
		// and in 'an' we have the angle, between the arrow line and
		// one of the coordinate axis
		float length = polarInfo.radius;
		float angle = polarInfo.angle;

		// Find two points around the destination control point
		//Convert.dekartToPolar(b, a, an, r);
		Convert.cartesianToPolar(b, a, polarInfo);
		Convert.polarToCartesian(b, polarInfo.angle + 105, 1.2f, p3);
		Convert.polarToCartesian(b, polarInfo.angle - 105, 1.2f, p4);

		// Create the graphics path containing the arrow rectangle
		GeneralPath path = new GeneralPath(GeneralPath.WIND_EVEN_ODD);
		//path.AddLines(pts);
		path.append(new Line2D.Float(p1, p2), true);
		path.append(new Line2D.Float(p2, p3), true);
		path.append(new Line2D.Float(p3, p4), true);
		path.append(new Line2D.Float(p4, p1), true);
		path.closePath();

		// fill the arrow with white color, or the color
		// of the destination box, if the arrow's type is gradient
		e.getGraphics().setColor(type.equals(LinkFillType.GRADIENT) ? dc : Color.white);
		e.getGraphics().fill(path);

		// The frame pen
		Pen frame = e.getLink().getEffectivePen();

		// We have to calculate the progress bar
		float progress = e.getLink().getWeight();

		// Calculate the progress bar length
		float progLen = progress * length / 100;

		// Now we have to find two points which
		// define the end of the progress bar
		// depending on the progress length
		Point2D.Float pp1 = new Point2D.Float();
		Point2D.Float pp2 = new Point2D.Float();
		Point2D.Float pp = new Point2D.Float();

		Convert.polarToCartesian(a, angle, progLen, pp);
		Convert.polarToCartesian(pp, angle - 105, 1.2f, pp1);
		Convert.polarToCartesian(pp, angle + 105, 1.2f, pp2);

		// Now depending on the arrow type perform some specific
		// drawing
		if (type.equals(LinkFillType.FLAT))
		{
			// Create the graphics path containing the rectangle of
			// the progress bar
			GeneralPath ppath = new GeneralPath();
			//ppath.AddLines(new PointF[]{pp1, pp2, p1, p2});
			ppath.append(new Line2D.Float(pp1, pp2), true);
			ppath.append(new Line2D.Float(pp2, p1), true);
			ppath.append(new Line2D.Float(p1, p2), true);
			ppath.append(new Line2D.Float(p2, pp1), true);
			ppath.closePath();

			// Fill the path with the color of the origin box
			e.getGraphics().setColor(oc);
			e.getGraphics().fill(ppath);
			
			// Frame the progress bar
			applyPenToGraphics(e.getGraphics(), frame);
			e.getGraphics().draw(ppath);

			//ppath.dispose();
		}
		else if (type.equals(LinkFillType.ARROW))
		{
			// Find the arrow point
			Point2D.Float arrowPoint = new Point2D.Float();
			Convert.polarToCartesian(a, angle, progLen + 3, arrowPoint);

			// Create the graphics path containing the rectangle of
			// the progress bar
			GeneralPath ppath = new GeneralPath();
			ppath.append(new Line2D.Float(pp1, arrowPoint), true);
			ppath.append(new Line2D.Float(arrowPoint, pp2), true);
			ppath.append(new Line2D.Float(pp2, p1), true);
			ppath.append(new Line2D.Float(p1, p2), true);
			ppath.append(new Line2D.Float(p2, pp1), true);
			//ppath.AddLines(new PointF[]{pp1, arrowPoint, pp2, p1, p2});

			// Fill the path with the color of the origin box
			e.getGraphics().setColor(oc);
			e.getGraphics().fill(ppath);
			
			// Frame the progress bar
			applyPenToGraphics(e.getGraphics(), frame);
			e.getGraphics().draw(ppath);

			//ppath.Dispose();
		}
		else if (type.equals(LinkFillType.GRADIENT))
		{
			// Create the graphics path containing the rectangle of
			// the progress bar
			GeneralPath ppath = new GeneralPath();
			//ppath.AddLines(new PointF[]{pp1, pp2, p1, p2});
			ppath.append(new Line2D.Float(pp1, pp2), true);
			ppath.append(new Line2D.Float(pp2, p1), true);
			ppath.append(new Line2D.Float(p1, p2), true);
			ppath.append(new Line2D.Float(p2, pp1), true);
			

			// Fill the path with the gradient color
			applyGradientPaintToGraphics(e.getGraphics(), a, pp, oc, dc);
			e.getGraphics().fill(ppath);
		}
		
		// Draw the arrow frame using the arrow's pen settings
		applyPenToGraphics(e.getGraphics(), frame);
		e.getGraphics().draw(path);

		// Note: Instead of framing the whole rectangle we can only
		// draw the two parallel lines (results in cool effect).
		// To see it, comment the above line of code and
		// uncomment the two lines below
		// e.Graphics.DrawLine(frame, p1, p4);
		// e.Graphics.DrawLine(frame, p2, p3);

		// Dispose the GDI objects
		e.getGraphics().setPaint(paint);
	}
	
	void applyPenToGraphics(Graphics2D g, Pen p)
	{
		g.setPaint(p.getColor());
		g.setStroke(new BasicStroke(p.getWidth(), BasicStroke.CAP_ROUND,
			BasicStroke.JOIN_MITER, 1.0f));
	}

	void applyGradientPaintToGraphics(Graphics2D g,
		Point2D.Float p1, Point2D.Float p2, 
		Color color1, Color color2)
	{
		GradientPaint paint = new GradientPaint(
			p1.x, p1.y, color1, p2.x, p2.y, color2, true);
		g.setPaint(paint);
	}
	
	ActionListener taskPerformer = new ActionListener()
	{
		public void actionPerformed(ActionEvent evt)
		{
			for (DiagramLink link : diagram.getLinks())
			{
				link.setWeight( link.getWeight() + _r.nextInt(5));
				if (link.getWeight() > 100)
					link.setWeight(100);
			}

			diagramView.repaint();
	    }
	};
	
	private void initFrame()
	{
		diagram = new Diagram();
		diagramView = new DiagramView(diagram);
		diagramView.setBehavior(Behavior.DrawLinks);
		scrPane = new JScrollPane(diagramView);
		
		JPanel panel = new JPanel();
		panel.setLayout(null);

		JLabel label = new JLabel("Type of new links");
		label.setBounds(5, 10, 100, 23);
		
		cmBxLinkShape = new JComboBox(new Object[]{"Flat", "Arrow", "Gradient"});
		cmBxLinkShape.setBounds(105, 8, 100, 25);

		JButton btnAnimate = new JButton("Animate progress");
		btnAnimate.setBounds(30, 40, 140, 25);
		btnAnimate.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0)
			{
				animate();
			}
		});
		
		panel.add(label);
		panel.add(cmBxLinkShape);
		panel.add(btnAnimate);

		JTextArea txArea = new JTextArea();
		txArea.setText("This sample demonstrates custom painting of links." + 
				" To try it, create some nodes by clicking in the flowchart area above," + 
				" draw links between the nodes and click'animate'.");
		txArea.setWrapStyleWord(true);
		txArea.setEditable(false);
		txArea.setLineWrap(true);
		
		Container cp = this.getContentPane();
		SpringLayout layout = new SpringLayout();
		layout.putConstraint(SpringLayout.EAST, scrPane, 0, SpringLayout.EAST, cp);
		layout.putConstraint(SpringLayout.SOUTH, scrPane, -80, SpringLayout.SOUTH, cp);
		layout.putConstraint(SpringLayout.WEST, scrPane, 0, SpringLayout.WEST, cp);
		layout.putConstraint(SpringLayout.NORTH, scrPane, 0, SpringLayout.NORTH, cp);

		layout.putConstraint(SpringLayout.EAST, txArea, 0, SpringLayout.EAST, cp);
		layout.putConstraint(SpringLayout.SOUTH, txArea, 0, SpringLayout.SOUTH, cp);
		layout.putConstraint(SpringLayout.WEST, txArea, 230, SpringLayout.WEST, cp);
		layout.putConstraint(SpringLayout.NORTH, txArea, 0, SpringLayout.SOUTH, scrPane);
		
		layout.putConstraint(SpringLayout.EAST, panel, 0, SpringLayout.WEST, txArea);
		layout.putConstraint(SpringLayout.SOUTH, panel, 0, SpringLayout.SOUTH, cp);
		layout.putConstraint(SpringLayout.WEST, panel, 0, SpringLayout.WEST, cp);
		layout.putConstraint(SpringLayout.NORTH, panel, 0, SpringLayout.SOUTH, scrPane);

		this.setLayout(layout);
		this.add(panel);
		this.add(scrPane);
		this.add(txArea);

		this.setVisible(true);
		this.setSize(700, 600);
		
		this.addWindowListener(new WindowAdapter(){
			  public void windowClosing(WindowEvent arg0)
			  {
				  System.exit(0);
			  }
		});
		
		timer = new Timer(200, this.taskPerformer);
	}

	int mm = 0;
	private Diagram diagram;
	private DiagramView diagramView;

	private JScrollPane scrPane;
	private JComboBox cmBxLinkShape;
	
	private Timer timer;
	private Random _r = new Random();
	
	private static final long serialVersionUID = 1L;
}

/// <summary>
/// Defines the custom-drawn arrow type.
/// </summary>
class LinkFillType
{
	final static String FLAT = "FLAT";
	final static String ARROW = "ARROW";
	final static String GRADIENT = "GRADIENT";
}

class PolarSystemInfo
{
	public float angle;
	public float radius;
}

/// <summary>
/// Provides various conversions.
/// </summary>
class Convert
{
	/// <summary>
	/// Converts polar coordinates to the corresponding
	/// Cartesian coordinates, using the specified point as
	/// a center of the coordinate system.
	/// </summary>
	public static void polarToCartesian(Point2D.Float coordCenter,
		float a, float r, Point2D.Float dekart)
	{
		if (r == 0)
		{
			dekart = (Point2D.Float)coordCenter.clone();
			return;
		}

		dekart.x = (float)(coordCenter.getX() + Math.cos(a * Math.PI / 180) * r);
		dekart.y = (float)(coordCenter.getY() - Math.sin(a * Math.PI / 180) * r);
	}

	/// <summary>
	/// Converts Cartesian coordinates to the corresponding
	/// polar coordinates, using the specified point as
	/// a center of the coordinate system.
	/// </summary>
	public static void cartesianToPolar(Point2D.Float coordCenter,
		Point2D.Float dekart, PolarSystemInfo polarInfo)
	{
		if (coordCenter.getX() == dekart.getX() && coordCenter.getY() == dekart.getY())
		{
			polarInfo.angle = 0f;
			polarInfo.radius = 0f;
			return;
		}

		float dx = dekart.x - coordCenter.x;
		float dy = dekart.y - coordCenter.y;
		polarInfo.radius = (float)Math.sqrt(Math.pow(dx, 2) + Math.pow(dy, 2));

		polarInfo.angle = Float.valueOf((float)(Math.atan(-dy / dx) * 180 / Math.PI));
		if (dx < 0) 
			polarInfo.angle = Float.valueOf((float)(polarInfo.angle + 180));
	}
}